@@ -216,7 +216,7 @@ namespace gensios {
 		    va_end(argcopy);
 		    std::string outstr(len + 1, '\0');
 		    vsnprintf(&outstr[0], len + 1, d->log, d->args);
-		    cb->parmlog(outstr);
+		    cb->parmlog(std::move(outstr));
 		    return 0;
 		}
 
@@ -332,13 +332,6 @@ namespace gensios {
 	}
     }
 
-    void
-    Serial_Gensio::set_gensio(struct gensio *io, bool set_cb)
-    {
-	this->sio = gensio_to_sergensio(io);
-	Gensio::set_gensio(io, set_cb);
-    }
-
     Gensio *
     gensio_alloc(struct gensio *io, Os_Funcs &o)
     {
@@ -353,12 +346,7 @@ namespace gensios {
 	for (i = 0; (cio = gensio_get_child(io, i)); i++) {
 	    if (gensio_get_frdata(cio))
 		break; // It's already been set.
-	    sio = gensio_to_sergensio(cio);
-	    if (sio) {
-		g = new Serial_Gensio(o, NULL);
-	    } else {
-		g = new Gensio(o, NULL);
-	    }
+	    g = new Gensio(o, NULL);
 	    g->set_gensio(cio, i == 0);
 	}
 	f = gensio_get_frdata(io);
@@ -406,7 +394,7 @@ namespace gensios {
 	    va_end(argcopy);
 	    std::string outstr(len, '\0');
 	    vsnprintf(&outstr[0], len + 1, p->log, p->args);
-	    cb->log(p->level, outstr);
+	    cb->log(p->level, std::move(outstr));
 	    return 0;
 	}
 
@@ -726,431 +714,6 @@ namespace gensios {
 	return 0;
     }
 
-    static void sergensio_cpp_done(struct sergensio *sio, int err,
-			    unsigned int val, void *cb_data)
-    {
-	if (!cb_data)
-	    return;
-	Serial_Op_Done *done = static_cast<Serial_Op_Done *>(cb_data);
-
-	done->serial_op_done(err, val);
-    }
-
-    void Serial_Gensio::flush(unsigned int flush)
-    {
-	int err;
-
-	err = sergensio_flush(sio, flush);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::baud(unsigned int baud, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_baud(sio, baud, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::datasize(unsigned int size, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_datasize(sio, size, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::parity(unsigned int par, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_parity(sio, par, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::stopbits(unsigned int bits, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_stopbits(sio, bits, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::flowcontrol(unsigned int flow, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_flowcontrol(sio, flow, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::iflowcontrol(unsigned int flow,
-					 Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_iflowcontrol(sio, flow, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::sbreak(unsigned int sbreak, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_sbreak(sio, sbreak, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::dtr(unsigned int dtr, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_dtr(sio, dtr, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::rts(unsigned int rts, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_rts(sio, rts, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::cts(unsigned int cts, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_cts(sio, cts, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::dcd_dsr(unsigned int dcd_dsr, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_dcd_dsr(sio, dcd_dsr, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::ri(unsigned int ri, Serial_Op_Done *done)
-    {
-	int err;
-	sergensio_done donefunc = sergensio_cpp_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_ri(sio, ri, donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    static void sergensio_cpp_sig_done(struct sergensio *sio, int err,
-				       const char *sig, unsigned int len,
-				       void *cb_data)
-    {
-	if (!cb_data)
-	    return;
-	Serial_Op_Sig_Done *done = static_cast<Serial_Op_Sig_Done *>(cb_data);
-	std::vector<unsigned char> sigv(sig, sig + len);
-
-	done->serial_op_sig_done(err, std::move(sigv));
-    }
-
-    void Serial_Gensio::signature(const std::vector<unsigned char> sig,
-				  Serial_Op_Sig_Done *done)
-    {
-	int err;
-	sergensio_done_sig donefunc = sergensio_cpp_sig_done;
-
-	if (!done)
-	    donefunc = NULL;
-	err = sergensio_signature(sio, (const char *) sig.data(),
-				  (gensiods) sig.size(), donefunc, done);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    class Std_Ser_Op_Done: public Serial_Op_Done {
-    public:
-	Std_Ser_Op_Done(Os_Funcs &o) : waiter(o) { }
-
-	int wait(gensio_time *timeout = NULL, bool intr = false)
-	{
-	    return waiter.wait(1, timeout, intr);
-	}
-
-	int err = 0;
-	unsigned int val = 0;
-
-    private:
-	void serial_op_done(int err, unsigned int val)
-	{
-	    this->err = err;
-	    this->val = val;
-	    waiter.wake();
-	}
-	Waiter waiter;
-    };
-
-    int Serial_Gensio::baud_s(unsigned int *baud, gensio_time *timeout,
-			      bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->baud(*baud, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*baud = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::datasize_s(unsigned int *size, gensio_time *timeout,
-				  bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->datasize(*size, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*size = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::parity_s(unsigned int *par, gensio_time *timeout,
-				bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->parity(*par, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*par = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::stopbits_s(unsigned int *bits, gensio_time *timeout,
-				  bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->stopbits(*bits, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*bits = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::flowcontrol_s(unsigned int *flow, gensio_time *timeout,
-				     bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->flowcontrol(*flow, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*flow = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::iflowcontrol_s(unsigned int *flow, gensio_time *timeout,
-				      bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->iflowcontrol(*flow, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*flow = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::sbreak_s(unsigned int *sbreak, gensio_time *timeout,
-				bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->sbreak(*sbreak, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*sbreak = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::dtr_s(unsigned int *dtr, gensio_time *timeout,
-			     bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->dtr(*dtr, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*dtr = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::rts_s(unsigned int *rts, gensio_time *timeout,
-			     bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->rts(*rts, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*rts = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::cts_s(unsigned int *cts, gensio_time *timeout,
-			     bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->cts(*cts, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*cts = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::dcd_dsr_s(unsigned int *dcd_dsr, gensio_time *timeout,
-				 bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->dcd_dsr(*dcd_dsr, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*dcd_dsr = w.val;
-	return 0;
-    }
-
-    int Serial_Gensio::ri_s(unsigned int *ri, gensio_time *timeout,
-			    bool intr)
-    {
-	Std_Ser_Op_Done w(this->get_os_funcs());
-	int err;
-
-	this->ri(*ri, &w);
-	err = w.wait(timeout, intr);
-	if (err)
-	    return err;
-	if (w.err)
-	    throw gensio_error(w.err);
-	*ri = w.val;
-	return 0;
-    }
-
-    void Serial_Gensio::modemstate(unsigned int state)
-    {
-	int err = sergensio_modemstate(sio, state);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::linestate(unsigned int state)
-    {
-	int err = sergensio_linestate(sio, state);
-	if (err)
-	    throw gensio_error(err);
-    }
-
-    void Serial_Gensio::flow_state(bool state)
-    {
-	int err = sergensio_flowcontrol_state(sio, state);
-	if (err)
-	    throw gensio_error(err);
-    }
-
     struct gensio_acc_cpp_data {
 	struct gensio_acc_frdata frdata;
 	Accepter *a;
@@ -1183,7 +746,7 @@ namespace gensios {
 		    va_end(argcopy);
 		    std::string outstr(len + 1, '\0');
 		    vsnprintf(&outstr[0], len + 1, d->log, d->args);
-		    cb->parmlog(outstr);
+		    cb->parmlog(std::move(outstr));
 		    break;
 		}
 
