@@ -83,16 +83,17 @@ static void *
 gensio_glib_zalloc(struct gensio_os_funcs *f, gensiods size)
 {
     struct gensio_data *d = f->user_data;
-
-    return gensio_i_zalloc(d->mtrack, size);
+    TRACE_MEM;
+    return gensio_i_zalloc(d->mtrack, size,
+			   TRACE_MEM_CALLERS, TRACE_MEM_CALLERS_SIZE);
 }
 
 static void
 gensio_glib_free(struct gensio_os_funcs *f, void *data)
 {
     struct gensio_data *d = f->user_data;
-
-    gensio_i_free(d->mtrack, data);
+    TRACE_MEM;
+    gensio_i_free(d->mtrack, data, TRACE_MEM_CALLERS, TRACE_MEM_CALLERS_SIZE);
 }
 
 static void
@@ -161,6 +162,7 @@ struct gensio_iod_glib {
 
     int orig_fd;
     int fd;
+    int sfd;
     enum gensio_iod_type type;
     void *sockinfo;
     bool handlers_set;
@@ -1095,12 +1097,13 @@ gensio_glib_wake(struct gensio_waiter *w)
 
 static int
 gensio_glib_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
-		    intptr_t ofd, struct gensio_iod **riod)
+		    intptr_t ofd, struct gensio_iod **riod, ...)
 {
     struct gensio_iod_glib *iod;
     bool closefd = false;
     int err = GE_NOMEM;
     intptr_t fd = ofd;
+    int sfd = -1;
 
 #ifndef _WIN32
     if (type == GENSIO_IOD_CONSOLE) {
@@ -1115,7 +1118,7 @@ gensio_glib_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 	closefd = true;
     } else if (type == GENSIO_IOD_PTY) {
 	int ufd;
-	err = gensio_unix_pty_alloc(o, &ufd);
+	err = gensio_unix_pty_alloc(o, &ufd, &sfd);
 	if (err)
 	    return err;
 	fd = ufd;
@@ -1132,6 +1135,7 @@ gensio_glib_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
     iod->r.f = o;
     iod->fd = fd;
     iod->orig_fd = ofd;
+    iod->sfd = sfd;
     if (type == GENSIO_IOD_STDIO) {
 #ifndef _WIN32
 	struct stat statb;
@@ -1195,8 +1199,11 @@ gensio_glib_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 
  out_err:
     o->free(o, iod);
-    if (closefd)
+    if (closefd) {
 	close(fd);
+	if (sfd != -1)
+	    close(sfd);
+    }
     return err;
 }
 
@@ -1276,7 +1283,7 @@ gensio_glib_pty_control(struct gensio_iod_glib *iod, int op, bool get,
 #ifdef _WIN32
 	return GE_NOTSUP;
 #else
-	return gensio_unix_pty_start(o, iod->fd, iod->argv,
+	return gensio_unix_pty_start(o, iod->fd, &iod->sfd, iod->argv,
 				     iod->env, iod->start_dir, &iod->pid);
 #endif
 
@@ -1423,6 +1430,7 @@ gensio_glib_close(struct gensio_iod **iodp)
 #else
 	if (iod->fd != -1) {
 	    err = close(iod->fd);
+	    iod->fd = -1;
 	    if (err == -1)
 		err = gensio_os_err_to_err(o, errno);
 #endif
@@ -1431,6 +1439,10 @@ gensio_glib_close(struct gensio_iod **iodp)
 	    assert(err == 0);
 #endif
 	}
+	if (iod->sfd != -1) {
+	    err = close(iod->sfd);
+	    iod->sfd = -1;
+	}
     }
     o->release_iod(iiod);
     *iodp = NULL;
@@ -1926,22 +1938,6 @@ gensio_glib_free_funcs(struct gensio_os_funcs *f)
     free(f);
 }
 
-static GMutex once_lock;
-
-static void
-gensio_glib_call_once(struct gensio_os_funcs *f, struct gensio_once *once,
-		      void (*func)(void *cb_data), void *cb_data)
-{
-    if (once->called)
-	return;
-    g_mutex_lock(&once_lock);
-    if (!once->called) {
-	once->called = true;
-	func(cb_data);
-    }
-    g_mutex_unlock(&once_lock);
-}
-
 static void
 gensio_glib_get_monotonic_time(struct gensio_os_funcs *f, gensio_time *time)
 {
@@ -2053,7 +2049,7 @@ gensio_glib_funcs_alloc(struct gensio_os_funcs **ro)
     o->service = gensio_glib_service;
     o->get_funcs = gensio_glib_get_funcs;
     o->free_funcs = gensio_glib_free_funcs;
-    o->call_once = gensio_glib_call_once;
+    o->call_once = gensio_call_once;
     o->get_monotonic_time = gensio_glib_get_monotonic_time;
     o->handle_fork = gensio_glib_handle_fork;
     o->add_iod = gensio_glib_add_iod;
