@@ -27,42 +27,6 @@
 
 static void check_flush_sync_io(struct gensio *io);
 
-struct gensio_classobj {
-    const char *name;
-    void *classdata;
-    struct gensio_classops *ops;
-    struct gensio_classobj *next;
-};
-
-static int
-gen_addclass(struct gensio_os_funcs *o, struct gensio_classobj **classes,
-	     const char *name, struct gensio_classops *ops, void *classdata)
-{
-    struct gensio_classobj *c;
-
-    c = o->zalloc(o, sizeof(*c));
-    if (!c)
-	return GE_NOMEM;
-    c->name = name;
-    c->ops = ops;
-    c->classdata = classdata;
-    c->next = *classes;
-    *classes = c;
-    return 0;
-}
-
-static void *
-gen_getclass(struct gensio_classobj *classes, const char *name)
-{
-    struct gensio_classobj *c;
-
-    for (c = classes; c; c = c->next) {
-	if (strcmp(c->name, name) == 0)
-	    return c->classdata;
-    }
-    return NULL;
-}
-
 struct gensio_nocbwait {
     bool queued;
     struct gensio_waiter *waiter;
@@ -78,8 +42,6 @@ struct gensio {
     unsigned int refcount;
     struct gensio_lock *lock;
 
-    struct gensio_classobj *classes;
-
     gensio_func func;
     void *gensio_data;
 
@@ -168,21 +130,6 @@ gensio_data_alloc(struct gensio_os_funcs *o,
     io->gensio_data = gensio_data;
     io->child = child;
 
-    if (child) {
-	struct gensio_classobj *c = child->classes;
-
-	while (c) {
-	    if (c->ops && c->ops->propagate_to_parent) {
-		int rv = c->ops->propagate_to_parent(io, child, c->classdata);
-		if (rv) {
-		    gensio_data_free(io);
-		    return NULL;
-		}
-	    }
-	    c = c->next;
-	}
-    }
-
     o_base->lock(gensio_base_lock);
     num_alloced_gensios++;
     o_base->unlock(gensio_base_lock);
@@ -200,14 +147,6 @@ gensio_data_free(struct gensio *io)
     if (io->frdata && io->frdata->freed)
 	io->frdata->freed(io, io->frdata);
 
-    while (io->classes) {
-	struct gensio_classobj *c = io->classes;
-
-	if (c->ops && c->ops->cleanup)
-	    c->ops->cleanup(io, c->classdata);
-	io->classes = c->next;
-	io->o->free(io->o, c);
-    }
     io->o->free_lock(io->lock);
     io->o->free(io->o, io);
 
@@ -290,67 +229,6 @@ gensio_glog(struct gensio *io, enum gensio_log_levels level,
     va_end(args);
 }
 
-int
-gensio_addclass(struct gensio *io, const char *name, int classops_ver,
-		struct gensio_classops *ops, void *classdata)
-{
-    return gen_addclass(io->o, &io->classes, name, ops, classdata);
-}
-
-void *
-gensio_getclass(struct gensio *io, const char *name)
-{
-    struct gensio *c = io;
-    void *rv = NULL;
-
-    while (c) {
-	rv = gen_getclass(c->classes, name);
-	if (rv)
-	    return rv;
-	c = c->child;
-    }
-
-    return rv;
-}
-
-struct gensio_acc_classobj {
-    const char *name;
-    void *classdata;
-    struct gensio_acc_classops *ops;
-    struct gensio_acc_classobj *next;
-};
-
-static int
-gen_acc_addclass(struct gensio_os_funcs *o,
-		 struct gensio_acc_classobj **classes,
-		 const char *name, struct gensio_acc_classops *ops,
-		 void *classdata)
-{
-    struct gensio_acc_classobj *c;
-
-    c = o->zalloc(o, sizeof(*c));
-    if (!c)
-	return GE_NOMEM;
-    c->name = name;
-    c->ops = ops;
-    c->classdata = classdata;
-    c->next = *classes;
-    *classes = c;
-    return 0;
-}
-
-static void *
-gen_acc_getclass(struct gensio_acc_classobj *classes, const char *name)
-{
-    struct gensio_acc_classobj *c;
-
-    for (c = classes; c; c = c->next) {
-	if (strcmp(c->name, name) == 0)
-	    return c->classdata;
-    }
-    return NULL;
-}
-
 struct gensio_accepter {
     struct gensio_os_funcs *o;
 
@@ -358,8 +236,6 @@ struct gensio_accepter {
     gensio_accepter_event cb;
     struct gensio_lock *lock;
 
-    struct gensio_acc_classobj *classes;
-
     const struct gensio_accepter_functions *funcs;
     gensio_acc_func func;
     void *gensio_acc_data;
@@ -417,21 +293,6 @@ gensio_acc_data_alloc(struct gensio_os_funcs *o,
     gensio_list_init(&acc->waiting_ios);
     gensio_list_init(&acc->waiting_accepts);
 
-    if (child) {
-	struct gensio_acc_classobj *c = child->classes;
-
-	while (c) {
-	    if (c->ops && c->ops->propagate_to_parent) {
-		int rv = c->ops->propagate_to_parent(acc, child, c->classdata);
-		if (rv) {
-		    gensio_acc_data_free(acc);
-		    return NULL;
-		}
-	    }
-	    c = c->next;
-	}
-    }
-
     return acc;
 }
 
@@ -441,14 +302,6 @@ gensio_acc_data_free(struct gensio_accepter *acc)
     if (acc->frdata && acc->frdata->freed)
 	acc->frdata->freed(acc, acc->frdata);
 
-    while (acc->classes) {
-	struct gensio_acc_classobj *c = acc->classes;
-
-	if (c->ops && c->ops->cleanup)
-	    c->ops->cleanup(acc, c->classdata);
-	acc->classes = c->next;
-	acc->o->free(acc->o, c);
-    }
     if (acc->lock)
 	acc->o->free_lock(acc->lock);
     acc->o->free(acc->o, acc);
@@ -493,21 +346,6 @@ gensio_acc_cb(struct gensio_accepter *acc, int event, void *data)
     return acc->cb(acc, acc->user_data, event, data);
 }
 
-int
-gensio_acc_addclass(struct gensio_accepter *acc,
-		    const char *name, int classops_ver,
-		    struct gensio_acc_classops *ops,
-		    void *classdata)
-{
-    return gen_acc_addclass(acc->o, &acc->classes, name, ops, classdata);
-}
-
-void *
-gensio_acc_getclass(struct gensio_accepter *acc, const char *name)
-{
-    return gen_acc_getclass(acc->classes, name);
-}
-
 const char *
 gensio_acc_get_type(struct gensio_accepter *acc, unsigned int depth)
 {
@@ -2618,6 +2456,8 @@ struct gensio_def_entry builtin_defaults[] = {
 						.def.intval = 10 },
     /* TCP and SCTP, UDP get added in init as false. */
     { "reuseaddr",	GENSIO_DEFAULT_BOOL,	.def.intval = 1 },
+    { "drain_timeout",	GENSIO_DEFAULT_INT,	.min = -1, .max = INT_MAX,
+						.def.intval = -1, },
     /* serialdev */
     { "xonxoff",	GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
     { "rtscts",		GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
