@@ -7,7 +7,7 @@
 
 /* This code handles running a child process using a pty. */
 
-#ifdef linux
+#if defined(linux) || defined(__MSYS__)
 #define _GNU_SOURCE /* Get ptsname_r(). */
 #endif
 
@@ -78,7 +78,8 @@ struct pty_data {
     bool exit_code_set;
 };
 
-static int pty_check_open(void *handler_data, struct gensio_iod *iod)
+static int pty_check_open(void *handler_data, struct gensio_iod *iod,
+			  gensio_time *timeout)
 {
     return 0;
 }
@@ -307,7 +308,8 @@ gensio_setup_child_on_pty(struct pty_data *tdata)
 }
 
 static int
-pty_sub_open(void *handler_data, struct gensio_iod **riod)
+pty_sub_open(void *handler_data, struct gensio_iod **riod,
+	     gensio_time *timeout)
 {
     struct pty_data *tdata = handler_data;
     int err;
