@@ -18,6 +18,11 @@
 #else
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <unistd.h>
+#ifdef __MSYS__
+typedef struct in6_addr IN6_ADDR;
+#include <iphlpapi.h>
+#endif
 #endif
 #include <gensio/gensio.h>
 #include <gensio/gensio_list.h>
@@ -3034,6 +3039,9 @@ win_browse_query_complete(void *context,
     }
 
  out_err:
+    if (err)
+	gensio_log(o, GENSIO_LOG_INFO, "Invalid mdns data: %s",
+		   gensio_err_to_str(err));
     if (name)
 	o->free(o, name);
     if (host)
@@ -3406,8 +3414,6 @@ gensio_mdnslib_start(struct gensio_mdns *m)
 	o->clear_fd_handlers_norpt(m->iod);
     if (m->dnssd_fd != -1)
 	DNSServiceRefDeallocate(m->dnssd_sref);
-    if (m->lock)
-	o->free_lock(m->lock);
     return err;
 }
 
