@@ -14,11 +14,12 @@
 
 #include "utils.h"
 
+#include <gensio/gensio.h>
 #include <gensio/gensio_err.h>
 #include <gensio/gensio_class.h>
-#include <gensio/sergensio_class.h>
 #include <gensio/gensio_base.h>
 #include <gensio/gensio_list.h>
+#include <gensio/gensio_openipmi_oshandler.h>
 
 #include <gensio/gensio_buffer.h>
 #include "utils.h"
@@ -40,500 +41,6 @@
  */
 typedef void (*gensio_ll_ipmisol_cb)(void *handler_data, int op, void *data);
 
-/* op is values from sergensio_class.h. */
-typedef int (*gensio_ll_ipmisol_ops)(struct gensio_ll *ll, int op,
-				     int val, char *buf,
-				     void *done, void *cb_data);
-struct igensio_info
-{
-    struct gensio_os_funcs *o;
-    os_vlog_t log_handler;
-};
-
-
-static void *
-gio_mem_alloc(int size)
-{
-    return malloc(size);
-}
-
-static void
-gio_mem_free(void *data)
-{
-    free(data);
-}
-
-struct os_hnd_fd_id_s
-{
-    struct gensio_os_funcs *o;
-    int             fd;
-    struct gensio_iod *iod;
-    void            *cb_data;
-    os_data_ready_t data_ready;
-    os_data_ready_t write_ready;
-    os_data_ready_t except_ready;
-    os_handler_t    *handler;
-    os_fd_data_freed_t freed;
-};
-
-static void
-fd_read_handler(struct gensio_iod *iod, void *data)
-{
-    os_hnd_fd_id_t *fd_data = (os_hnd_fd_id_t *) data;
-
-    fd_data->data_ready(fd_data->fd, fd_data->cb_data, fd_data);
-}
-
-static void
-fd_write_handler(struct gensio_iod *iod, void *data)
-{
-    os_hnd_fd_id_t *fd_data = (os_hnd_fd_id_t *) data;
-
-    fd_data->write_ready(fd_data->fd, fd_data->cb_data, fd_data);
-}
-
-static void
-fd_except_handler(struct gensio_iod *iod, void *data)
-{
-    os_hnd_fd_id_t *fd_data = (os_hnd_fd_id_t *) data;
-
-    fd_data->except_ready(fd_data->fd, fd_data->cb_data, fd_data);
-}
-
-static void
-free_fd_data(struct gensio_iod *iod, void *data)
-{
-    os_hnd_fd_id_t *fd_data = data;
-
-    fd_data->o->release_iod(fd_data->iod);
-    if (fd_data->freed)
-        fd_data->freed(fd_data->fd, fd_data->cb_data);
-    free(data);
-}
-
-static int
-gio_add_fd_to_wait_for(os_handler_t       *handler,
-		       int                fd,
-		       os_data_ready_t    data_ready,
-		       void               *cb_data,
-		       os_fd_data_freed_t freed,
-		       os_hnd_fd_id_t     **id)
-{
-    os_hnd_fd_id_t *fd_data;
-    int rv;
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-
-
-    fd_data = malloc(sizeof(*fd_data));
-    if (!fd_data)
-	return ENOMEM;
-    fd_data->o = o;
-
-    rv = o->add_iod(o, GENSIO_IOD_SOCKET, fd, &fd_data->iod);
-    if (rv) {
-	free(fd_data);
-	return rv;
-    }
-    fd_data->fd = fd;
-    fd_data->cb_data = cb_data;
-    fd_data->data_ready = data_ready;
-    fd_data->handler = handler;
-    fd_data->freed = freed;
-    rv = o->set_fd_handlers(fd_data->iod, fd_data,
-			    fd_read_handler, fd_write_handler,
-			    fd_except_handler, free_fd_data);
-    if (rv) {
-	o->release_iod(fd_data->iod);
-	free(fd_data);
-	return rv;
-    }
-    o->set_write_handler(fd_data->iod, false);
-    o->set_except_handler(fd_data->iod, false);
-    o->set_read_handler(fd_data->iod, true);
-
-    *id = fd_data;
-    return 0;
-}
-
-static int
-gio_remove_fd_to_wait_for(os_handler_t   *handler,
-			  os_hnd_fd_id_t *id)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-
-    o->set_read_handler(id->iod, false);
-    o->clear_fd_handlers(id->iod);
-    return 0;
-}
-
-struct os_hnd_timer_id_s
-{
-    void           *cb_data;
-    os_timed_out_t timed_out;
-    struct gensio_timer *timer;
-    bool running;
-    os_handler_t *handler;
-    struct gensio_lock *lock;
-};
-
-static void
-timer_handler(struct gensio_timer *t, void *data)
-{
-    os_hnd_timer_id_t *timer = (os_hnd_timer_id_t *) data;
-    os_handler_t *handler = timer->handler;
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-    void              *cb_data;
-    os_timed_out_t    timed_out;
-
-    o->lock(timer->lock);
-    timed_out = timer->timed_out;
-    cb_data = timer->cb_data;
-    timer->running = false;
-    o->unlock(timer->lock);
-    timed_out(cb_data, timer);
-}
-
-static int
-gio_alloc_timer(os_handler_t      *handler,
-		os_hnd_timer_id_t **rtimer)
-{
-    os_hnd_timer_id_t *timer;
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-
-    timer = malloc(sizeof(*timer));
-    if (!timer)
-	return ENOMEM;
-
-    timer->lock = o->alloc_lock(o);
-    if (!timer->lock) {
-	free(timer);
-	return ENOMEM;
-    }
-
-    timer->running = false;
-    timer->timed_out = NULL;
-    timer->handler = handler;
-
-    timer->timer = o->alloc_timer(o, timer_handler, timer);
-    if (!timer->timer) {
-	o->free_lock(timer->lock);
-	free(timer);
-	return ENOMEM;
-    }
-
-    *rtimer = timer;
-    return 0;
-}
-
-static int
-gio_free_timer(os_handler_t      *handler,
-	       os_hnd_timer_id_t *timer)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-
-    o->free_timer(timer->timer);
-    o->free_lock(timer->lock);
-    free(timer);
-    return 0;
-}
-
-static int
-gio_start_timer(os_handler_t      *handler,
-		os_hnd_timer_id_t *timer,
-		struct timeval    *timeout,
-		os_timed_out_t    timed_out,
-		void              *cb_data)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-    gensio_time gtime;
-    int rv = 0;
-
-    o->lock(timer->lock);
-    if (timer->running) {
-	rv = EAGAIN;
-	goto out_unlock;
-    }
-
-    timer->running = true;
-    timer->cb_data = cb_data;
-    timer->timed_out = timed_out;
-
-    gtime.secs = timeout->tv_sec;
-    gtime.nsecs = timeout->tv_usec * 1000;
-
-    rv = o->start_timer(timer->timer, &gtime);
-    if (rv)
-	timer->running = false;
-
- out_unlock:
-    o->unlock(timer->lock);
-
-    return rv;
-}
-
-static int
-gio_stop_timer(os_handler_t *handler,
-	       os_hnd_timer_id_t *timer)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-    int rv = 0;
-
-    o->lock(timer->lock);
-    if (timer->running) {
-	timer->running = 0;
-	o->stop_timer(timer->timer);
-    } else {
-	rv = ETIMEDOUT;
-    }
-    o->unlock(timer->lock);
-
-    return rv;
-}
-
-struct os_hnd_lock_s
-{
-    struct gensio_lock *lock;
-};
-
-static int
-gio_create_lock(os_handler_t  *handler,
-		os_hnd_lock_t **rlock)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-    os_hnd_lock_t *lock;
-
-    lock = malloc(sizeof(*lock));
-    if (!lock)
-	return ENOMEM;
-
-    lock->lock = o->alloc_lock(o);
-    if (!lock->lock) {
-	free(lock);
-	return ENOMEM;
-    }
-
-    *rlock = lock;
-
-    return 0;
-}
-
-static int
-gio_destroy_lock(os_handler_t  *handler,
-		 os_hnd_lock_t *lock)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-
-    o->free_lock(lock->lock);
-    free(lock);
-    return 0;
-}
-
-static int
-gio_lock(os_handler_t  *handler,
-	 os_hnd_lock_t *lock)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-
-    o->lock(lock->lock);
-    return 0;
-}
-
-static int
-gio_unlock(os_handler_t  *handler,
-	   os_hnd_lock_t *lock)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-
-    o->unlock(lock->lock);
-    return 0;
-}
-
-static int
-gio_get_random(os_handler_t  *handler,
-	       void          *data,
-	       unsigned int  len)
-{
-    struct igensio_info *info = handler->internal_data;
-
-    return info->o->get_random(info->o, data, len);
-}
-
-static void
-gio_vlog(os_handler_t         *handler,
-	 enum ipmi_log_type_e log_type,
-	 const char           *format,
-	 va_list              ap)
-{
-    struct igensio_info *info = handler->internal_data;
-    os_vlog_t log_handler = info->log_handler;
-    enum gensio_log_levels level;
-
-    switch(log_type) {
-    case IPMI_LOG_INFO:
-    default:
-	level = GENSIO_LOG_INFO;
-	break;
-
-    case IPMI_LOG_WARNING:
-    case IPMI_LOG_ERR_INFO:
-	level = GENSIO_LOG_WARNING;
-	break;
-
-    case IPMI_LOG_SEVERE:
-	level = GENSIO_LOG_ERR;
-	break;
-
-    case IPMI_LOG_FATAL:
-	level = GENSIO_LOG_FATAL;
-	break;
-
-    case IPMI_LOG_DEBUG:
-    case IPMI_LOG_DEBUG_START:
-    case IPMI_LOG_DEBUG_CONT:
-    case IPMI_LOG_DEBUG_END:
-	level = GENSIO_LOG_DEBUG;
-	break;
-    }
-
-    if (log_handler) {
-	log_handler(handler, format, log_type, ap);
-    } else if (info->o->vlog) {
-	gensio_vlog(info->o, level, format, ap);
-    } else if (gensio_get_log_mask() & (1 << level)) {
-	vprintf(format, ap);
-	putc('\n', stdout);
-    }
-}
-
-static void
-gio_log(os_handler_t         *handler,
-	enum ipmi_log_type_e log_type,
-	const char           *format,
-	...)
-{
-    va_list ap;
-
-    va_start(ap, format);
-    gio_vlog(handler, log_type, format, ap);
-    va_end(ap);
-}
-
-static void
-gio_set_log_handler(os_handler_t *handler,
-		    os_vlog_t    log_handler)
-{
-    struct igensio_info *info = handler->internal_data;
-
-    info->log_handler = log_handler;
-}
-
-static void
-gio_set_fd_handlers(os_handler_t *handler, os_hnd_fd_id_t *id,
-		    os_data_ready_t write_ready,
-		    os_data_ready_t except_ready)
-{
-    id->write_ready = write_ready;
-    id->except_ready = except_ready;
-}
-
-static int
-gio_set_fd_enables(os_handler_t *handler, os_hnd_fd_id_t *id,
-		   int read, int write, int except)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-
-    o->set_read_handler(id->iod, read);
-    o->set_write_handler(id->iod, write);
-    o->set_except_handler(id->iod, except);
-    return 0;
-}
-
-static int
-gio_get_monotonic_time(os_handler_t *handler, struct timeval *tv)
-{
-    struct igensio_info *info = handler->internal_data;
-    struct gensio_os_funcs *o = info->o;
-    gensio_time gtime;
-
-    o->get_monotonic_time(o, &gtime);
-    tv->tv_sec = gtime.secs;
-    tv->tv_usec = (gtime.nsecs + 500) / 1000;
-    return 0;
-}
-
-static int
-gio_get_real_time(os_handler_t *handler, struct timeval *tv)
-{
-    gettimeofday(tv, NULL);
-    return 0;
-}
-
-void ipmi_malloc_init(os_handler_t *oshandler);
-void ipmi_malloc_shutdown(void);
-
-static os_handler_t *
-gio_alloc(struct gensio_os_funcs *o)
-{
-    struct igensio_info *info;
-    os_handler_t *handler;
-    os_handler_t dummyh;
-
-    info = malloc(sizeof(*info));
-    if (!info)
-	return NULL;
-    info->o = o;
-    info->log_handler = NULL;
-
-    memset(&dummyh, 0, sizeof(dummyh));
-    dummyh.mem_alloc = gio_mem_alloc;
-    dummyh.mem_free = gio_mem_free;
-    ipmi_malloc_init(&dummyh);
-
-    handler = ipmi_alloc_os_handler();
-    if (!handler) {
-	free(info);
-	return NULL;
-    }
-
-    ipmi_malloc_shutdown();
-
-    handler->mem_alloc = gio_mem_alloc;
-    handler->mem_free = gio_mem_free;
-    handler->add_fd_to_wait_for = gio_add_fd_to_wait_for;
-    handler->remove_fd_to_wait_for = gio_remove_fd_to_wait_for;
-    handler->alloc_timer = gio_alloc_timer;
-    handler->free_timer = gio_free_timer;
-    handler->start_timer = gio_start_timer;
-    handler->stop_timer = gio_stop_timer;
-    handler->create_lock = gio_create_lock;
-    handler->destroy_lock = gio_destroy_lock;
-    handler->lock = gio_lock;
-    handler->unlock = gio_unlock;
-    handler->get_random = gio_get_random;
-    handler->log = gio_log;
-    handler->vlog = gio_vlog;
-    handler->set_log_handler = gio_set_log_handler;
-    handler->set_fd_handlers = gio_set_fd_handlers;
-    handler->set_fd_enables = gio_set_fd_enables;
-    handler->get_monotonic_time = gio_get_monotonic_time;
-    handler->get_real_time = gio_get_real_time;
-
-    handler->internal_data = info;
-    return handler;
-};
-
 enum sol_state {
     SOL_CLOSED,
     SOL_IN_OPEN,
@@ -555,7 +62,6 @@ struct sol_op_done {
     bool use_runner;
     gensio_control_done cdone;
     struct sol_xlat_str *xlatstr;
-    sergensio_done cb;
     int done_val;
     int val;
     void *cb_data;
@@ -567,7 +73,6 @@ struct sol_ll {
     struct gensio_ll *ll;
     struct gensio_os_funcs *o;
     struct gensio *io;
-    struct sergensio *sio;
 
     struct gensio_lock *lock;
 
@@ -659,7 +164,7 @@ struct sol_tc {
     struct gensio_link link;
 };
 
-os_handler_t *gensio_os_handler;
+static os_handler_t *gensio_os_handler;
 
 #define ll_to_sol(v) ((struct sol_ll *) gensio_ll_get_user_data(v))
 
@@ -1397,14 +902,11 @@ static void sol_disable(struct gensio_ll *ll)
 static int ipmisol_do_break(struct gensio_ll *ll);
 static int ipmisol_do_flush(struct gensio_ll *ll, int val, const char *sval);
 static int ipmisol_do_cts(struct gensio_ll *ll, int ival, const char *sval,
-			  gensio_control_done cdone,
-			  sergensio_done done, void *cb_data);
+			  gensio_control_done cdone, void *cb_data);
 static int ipmisol_do_dcd_dsr(struct gensio_ll *ll, int ival, const char *sval,
-			      gensio_control_done cdone,
-			      sergensio_done done, void *cb_data);
+			      gensio_control_done cdone, void *cb_data);
 static int ipmisol_do_ri(struct gensio_ll *ll, int ival, const char *sval,
-			 gensio_control_done cdone,
-			 sergensio_done done, void *cb_data);
+			 gensio_control_done cdone, void *cb_data);
 
 static int
 sol_control(struct gensio_ll *ll, bool get, unsigned int option,
@@ -1444,15 +946,15 @@ sol_acontrol(struct gensio_ll *ll, bool get, unsigned int option,
 
     switch (option) {
     case GENSIO_ACONTROL_SER_CTS:
-	return ipmisol_do_cts(ll, 0, data, idata->done, NULL,
+	return ipmisol_do_cts(ll, 0, data, idata->done,
 			      idata->cb_data);
 
     case GENSIO_ACONTROL_SER_DCD_DSR:
-	return ipmisol_do_dcd_dsr(ll, 0, data, idata->done, NULL,
+	return ipmisol_do_dcd_dsr(ll, 0, data, idata->done,
 				  idata->cb_data);
 
     case GENSIO_ACONTROL_SER_RI:
-	return ipmisol_do_ri(ll, 0, data, idata->done, NULL,
+	return ipmisol_do_ri(ll, 0, data, idata->done,
 			     idata->cb_data);
 
     default:
@@ -1533,25 +1035,25 @@ static int ipmisol_do_flush(struct gensio_ll *ll, int val, const char *sval)
 
     if (sval) {
 	if (strcmp(sval, "recv") == 0)
-	    val = SERGENSIO_FLUSH_RCV_BUFFER;
+	    val = GENSIO_SER_FLUSH_RECV;
 	else if (strcmp(sval, "xmit") == 0)
-	    val = SERGENSIO_FLUSH_XMIT_BUFFER;
+	    val = GENSIO_SER_FLUSH_XMIT;
 	else if (strcmp(sval, "both") == 0)
-	    val = SERGENSIO_FLUSH_RCV_XMIT_BUFFERS;
+	    val = GENSIO_SER_FLUSH_BOTH;
 	else
 	    return GE_INVAL;
     }
 
     switch(val) {
-    case SERGENSIO_FLUSH_RCV_BUFFER:
+    case GENSIO_SER_FLUSH_RECV:
 	val = IPMI_SOL_BMC_RECEIVE_QUEUE;
 	break;
 
-    case SERGENSIO_FLUSH_XMIT_BUFFER:
+    case GENSIO_SER_FLUSH_XMIT:
 	val = IPMI_SOL_BMC_TRANSMIT_QUEUE;
 	break;
 
-    case SERGENSIO_FLUSH_RCV_XMIT_BUFFERS:
+    case GENSIO_SER_FLUSH_BOTH:
 	return GE_NOTSUP;
 
     default:
@@ -1663,7 +1165,6 @@ sol_op_done(struct sol_ll *solll, int err, struct sol_op_done **op_done)
 {
     struct gensio_os_funcs *o = solll->o;
     struct sol_op_done *op = *op_done;
-    sergensio_done cb;
     gensio_control_done cdone;
     void *cb_data;
     const char *sval = NULL;
@@ -1674,7 +1175,6 @@ sol_op_done(struct sol_ll *solll, int err, struct sol_op_done **op_done)
     if (err)
 	err = sol_xlat_ipmi_err(solll->o, err);
 
-    cb = op->cb;
     cdone = op->cdone;
     val = op->done_val;
     if (!err && cdone) {
@@ -1700,10 +1200,6 @@ sol_op_done(struct sol_ll *solll, int err, struct sol_op_done **op_done)
 	sol_unlock(solll);
 	cdone(solll->io, err, sval, sval ? strlen(sval) : 0, cb_data);
 	sol_lock(solll);
-    } else if (cb) {
-	sol_unlock(solll);
-	cb(solll->sio, err, val, cb_data);
-	sol_lock(solll);
     }
     op = *op_done;
     if (op && !op->started) {
@@ -1719,7 +1215,7 @@ sol_do_op(struct sol_ll *solll, struct sol_op_done **op_done,
 		      void *),
 	  int val, int done_val, gensio_control_done cdone,
 	  struct sol_xlat_str *xlatstr,
-	  sergensio_done done, void *cb_data)
+	  void *cb_data)
 {
     struct gensio_os_funcs *o = solll->o;
     struct sol_op_done *op, *op2;
@@ -1731,7 +1227,6 @@ sol_do_op(struct sol_ll *solll, struct sol_op_done **op_done,
 
     op->use_runner = false;
     op->solll = solll;
-    op->cb = done;
     op->cb_data = cb_data;
     op->val = val;
     op->done_val = done_val;
@@ -1760,33 +1255,33 @@ sol_do_op(struct sol_ll *solll, struct sol_op_done **op_done,
 struct sol_xlat_str cts_xlat_str[] = {
     { "0", 0 },
     { "", 0 },
-    { "auto", SERGENSIO_CTS_AUTO },
-    { "off", SERGENSIO_CTS_OFF },
+    { "auto", GENSIO_SER_ON },
+    { "off", GENSIO_SER_OFF },
     {}
 };
 
 static int ipmisol_do_cts(struct gensio_ll *ll, int ival, const char *sval,
 			  gensio_control_done cdone,
-			  sergensio_done done, void *cb_data)
+			  void *cb_data)
 {
     struct sol_ll *solll = ll_to_sol(ll);
     int rv, val;
 
     if (sval) {
 	if (strcmp(sval, "auto") == 0)
-	    ival = SERGENSIO_CTS_AUTO;
+	    ival = GENSIO_SER_ON;
 	else if (strcmp(sval, "off") == 0)
-	    ival = SERGENSIO_CTS_OFF;
+	    ival = GENSIO_SER_OFF;
 	else
 	    return GE_INVAL;
     }
 
     sol_lock(solll);
     switch (ival) {
-    case SERGENSIO_CTS_AUTO:
+    case GENSIO_SER_ON:
 	val = 1;
 	break;
-    case SERGENSIO_CTS_OFF:
+    case GENSIO_SER_OFF:
 	val = 0;
 	break;
     default:
@@ -1794,7 +1289,7 @@ static int ipmisol_do_cts(struct gensio_ll *ll, int ival, const char *sval,
 	goto out_unlock;
     }
     rv = sol_do_op(solll, &solll->cts_done, ipmi_sol_set_CTS_assertable,
-		   val, ival, cdone, cts_xlat_str, done, cb_data);
+		   val, ival, cdone, cts_xlat_str, cb_data);
  out_unlock:
     sol_unlock(solll);
 
@@ -1810,27 +1305,26 @@ struct sol_xlat_str on_off_xlat_str[] = {
 };
 
 static int ipmisol_do_dcd_dsr(struct gensio_ll *ll, int ival, const char *sval,
-			      gensio_control_done cdone,
-			      sergensio_done done, void *cb_data)
+			      gensio_control_done cdone, void *cb_data)
 {
     struct sol_ll *solll = ll_to_sol(ll);
     int rv, val;
 
     if (sval) {
 	if (strcmp(sval, "on") == 0)
-	    ival = SERGENSIO_DCD_DSR_ON;
+	    ival = GENSIO_SER_ON;
 	else if (strcmp(sval, "off") == 0)
-	    ival = SERGENSIO_DCD_DSR_OFF;
+	    ival = GENSIO_SER_OFF;
 	else
 	    return GE_INVAL;
     }
 
     sol_lock(solll);
     switch (ival) {
-    case SERGENSIO_DCD_DSR_ON:
+    case GENSIO_SER_ON:
 	val = 1;
 	break;
-    case SERGENSIO_DCD_DSR_OFF:
+    case GENSIO_SER_OFF:
 	val = 0;
 	break;
     default:
@@ -1838,7 +1332,7 @@ static int ipmisol_do_dcd_dsr(struct gensio_ll *ll, int ival, const char *sval,
 	goto out_unlock;
     }
     rv = sol_do_op(solll, &solll->dcd_dsr_done, ipmi_sol_set_DCD_DSR_asserted,
-		   val, ival, cdone, on_off_xlat_str, done, cb_data);
+		   val, ival, cdone, on_off_xlat_str, cb_data);
  out_unlock:
     sol_unlock(solll);
 
@@ -1846,27 +1340,26 @@ static int ipmisol_do_dcd_dsr(struct gensio_ll *ll, int ival, const char *sval,
 }
 
 static int ipmisol_do_ri(struct gensio_ll *ll, int ival, const char *sval,
-			 gensio_control_done cdone,
-			 sergensio_done done, void *cb_data)
+			 gensio_control_done cdone, void *cb_data)
 {
     struct sol_ll *solll = ll_to_sol(ll);
     int rv, val;
 
     if (sval) {
 	if (strcmp(sval, "on") == 0)
-	    ival = SERGENSIO_RI_ON;
+	    ival = GENSIO_SER_ON;
 	else if (strcmp(sval, "off") == 0)
-	    ival = SERGENSIO_RI_OFF;
+	    ival = GENSIO_SER_OFF;
 	else
 	    return GE_INVAL;
     }
 
     sol_lock(solll);
     switch (ival) {
-    case SERGENSIO_RI_ON:
+    case GENSIO_SER_ON:
 	val = 1;
 	break;
-    case SERGENSIO_RI_OFF:
+    case GENSIO_SER_OFF:
 	val = 0;
 	break;
     default:
@@ -1874,53 +1367,13 @@ static int ipmisol_do_ri(struct gensio_ll *ll, int ival, const char *sval,
 	goto out_unlock;
     }
     rv = sol_do_op(solll, &solll->ri_done, ipmi_sol_set_RI_asserted,
-		   val, ival, cdone, on_off_xlat_str, done, cb_data);
+		   val, ival, cdone, on_off_xlat_str, cb_data);
  out_unlock:
     sol_unlock(solll);
 
     return rv;
 }
 
-static int
-ipmisol_ser_ops(struct gensio_ll *ll, int op,
-		int val, char *buf,
-		void *done, void *cb_data)
-{
-    switch (op) {
-    case SERGENSIO_FUNC_FLUSH:
-	return ipmisol_do_flush(ll, val, NULL);
-
-    case SERGENSIO_FUNC_SEND_BREAK:
-	return ipmisol_do_break(ll);
-
-    case SERGENSIO_FUNC_CTS:
-	return ipmisol_do_cts(ll, val, NULL, NULL, done, cb_data);
-
-    case SERGENSIO_FUNC_DCD_DSR:
-	return ipmisol_do_dcd_dsr(ll, val, NULL, NULL, done, cb_data);
-
-    case SERGENSIO_FUNC_RI:
-	return ipmisol_do_ri(ll, val, NULL, NULL, done, cb_data);
-
-    /* You really can't set much on a SOL connection once it's up. */
-    case SERGENSIO_FUNC_BAUD:
-    case SERGENSIO_FUNC_DATASIZE:
-    case SERGENSIO_FUNC_PARITY:
-    case SERGENSIO_FUNC_STOPBITS:
-    case SERGENSIO_FUNC_FLOWCONTROL:
-    case SERGENSIO_FUNC_IFLOWCONTROL:
-    case SERGENSIO_FUNC_SBREAK:
-    case SERGENSIO_FUNC_DTR:
-    case SERGENSIO_FUNC_RTS:
-    case SERGENSIO_FUNC_MODEMSTATE:
-    case SERGENSIO_FUNC_LINESTATE:
-    case SERGENSIO_FUNC_FLOWCONTROL_STATE:
-    case SERGENSIO_FUNC_SIGNATURE:
-    default:
-	return GE_NOTSUP;
-    }
-}
-
 static int
 sol_get_defaults(struct sol_ll *solll)
 {
@@ -2096,7 +1549,7 @@ gensio_ipmi_init(void *cb_data)
 {
     struct gensio_os_funcs *o = cb_data;
 
-    gensio_os_handler = gio_alloc(o);
+    gensio_os_handler = gensio_openipmi_oshandler_alloc(o);
     if (!gensio_os_handler)
 	abort();
     ipmi_init_err = ipmi_init(gensio_os_handler);
@@ -2112,7 +1565,6 @@ ipmisol_gensio_ll_alloc(struct gensio_pparm_info *p,
 			void *ser_cbs_data,
 			gensiods max_read_size,
 			gensiods max_write_size,
-			gensio_ll_ipmisol_ops *rops,
 			struct gensio_ll **rll)
 {
     struct sol_ll *solll;
@@ -2198,7 +1650,6 @@ ipmisol_gensio_ll_alloc(struct gensio_pparm_info *p,
     solll->ser_cbs = ser_cbs;
     solll->ser_cbs_data = ser_cbs_data;
 
-    *rops = ipmisol_ser_ops;
     *rll = solll->ll;
     return 0;
 
@@ -2209,31 +1660,16 @@ ipmisol_gensio_ll_alloc(struct gensio_pparm_info *p,
     return err;
 }
 
-static void
-ipmisol_gensio_ll_set_sio(struct gensio_ll *ll, struct sergensio *sio)
-{
-    struct sol_ll *solll = ll_to_sol(ll);
-
-    solll->sio = sio;
-    solll->io = sergensio_to_gensio(sio);
-}
-
-
 struct iterm_data {
-    struct sergensio *sio;
     struct gensio_os_funcs *o;
 
     struct gensio_ll *ll;
     struct gensio *io;
-
-    gensio_ll_ipmisol_ops ops;
 };
 
 static void
 iterm_free(struct iterm_data *idata)
 {
-    if (idata->sio)
-	sergensio_data_free(idata->sio);
     idata->o->free(idata->o, idata);
 }
 
@@ -2250,15 +1686,6 @@ iterm_ser_cb(void *handler_data, int op, void *data)
     gensio_cb(idata->io, op, 0, NULL, NULL, NULL);
 }
 
-static int
-sergensio_iterm_func(struct sergensio *sio, int op, int val, char *buf,
-		     void *done, void *cb_data)
-{
-    struct iterm_data *idata = sergensio_get_gensio_data(sio);
-
-    return idata->ops(idata->ll, op, val, buf, done, cb_data);
-}
-
 static int
 ipmisol_gensio_alloc(const void *gdata, const char * const args[],
 		     struct gensio_os_funcs *o,
@@ -2266,31 +1693,40 @@ ipmisol_gensio_alloc(const void *gdata, const char * const args[],
 		     struct gensio **rio)
 {
     const char *devname = gdata;
-    struct iterm_data *idata;
+    struct iterm_data *idata = NULL;
     int err;
     gensiods max_read_size = GENSIO_DEFAULT_BUF_SIZE;
     gensiods max_write_size = GENSIO_DEFAULT_BUF_SIZE;
     int i;
+    struct sol_ll *solll;
+    struct gensio_base_parms *parms = NULL;
     GENSIO_DECLARE_PPGENSIO(p, o, cb, "ipmisol", user_data);
 
+    err = gensio_base_parms_alloc(o, true, "ipmisol", &parms);
+    if (err)
+	goto out_err;
+
     for (i = 0; args && args[i]; i++) {
 	if (gensio_pparm_ds(&p, args[i], "readbuf", &max_read_size) > 0)
 	    continue;
 	if (gensio_pparm_ds(&p, args[i], "writebuf", &max_write_size) > 0)
 	    continue;
+	if (parms && gensio_base_parm(parms, &p, args[i]) > 0)
+	    continue;
 	gensio_pparm_unknown_parm(&p, args[i]);
-	return GE_INVAL;
+	err = GE_INVAL;
+	goto out_err;
     }
 
     idata = o->zalloc(o, sizeof(*idata));
     if (!idata)
-	return GE_NOMEM;
+	goto out_nomem;
 
     idata->o = o;
 
     err = ipmisol_gensio_ll_alloc(&p, o, devname, iterm_ser_cb, idata,
 				  max_read_size, max_write_size,
-				  &idata->ops, &idata->ll);
+				  &idata->ll);
     if (err)
 	goto out_err;
 
@@ -2298,25 +1734,30 @@ ipmisol_gensio_alloc(const void *gdata, const char * const args[],
 				  user_data);
     if (!idata->io) {
 	gensio_ll_free(idata->ll);
-	return GE_NOMEM;
+	goto out_nomem;
     }
 
-    gensio_set_is_serial(idata->io, true);
-
-    err = sergensio_addclass(o, idata->io, sergensio_iterm_func, idata,
-			     &idata->sio);
+    err = gensio_base_parms_set(idata->io, &parms);
     if (err) {
 	gensio_free(idata->io);
-	return err;
+	goto out_err;
     }
 
-    ipmisol_gensio_ll_set_sio(idata->ll, idata->sio);
+    solll = ll_to_sol(idata->ll);
+    solll->io = idata->io;
+
+    gensio_set_is_serial(idata->io, true);
 
     *rio = idata->io;
     return 0;
 
+ out_nomem:
+    err = GE_NOMEM;
  out_err:
-    iterm_free(idata);
+    if (idata)
+	iterm_free(idata);
+    if (parms)
+	gensio_base_parms_free(&parms);
     return err;
 }
 
