@@ -14,7 +14,6 @@
 #include <gensio/gensio.h>
 #include <gensio/gensio_base.h>
 #include <gensio/gensio_class.h>
-#include <gensio/sergensio_class.h>
 #include <gensio/gensio_ll_gensio.h>
 #include <gensio/gensio_acc_gensio.h>
 
@@ -846,6 +845,7 @@ gensio_telnet_filter_alloc(struct gensio_pparm_info *p,
 			   const struct gensio_telnet_filter_callbacks *cbs,
 			   void *handler_data,
 			   const struct gensio_telnet_filter_rops **rops,
+			   struct gensio_base_parms *parms,
 			   struct gensio_filter **rfilter)
 {
     struct gensio_filter *filter;
@@ -904,6 +904,8 @@ gensio_telnet_filter_alloc(struct gensio_pparm_info *p,
 	if (gensio_pparm_boolv(p, args[i], "mode", "client", "server",
 			       &is_client) > 0)
 	    continue;
+	if (parms && gensio_base_parm(parms, p, args[i]) > 0)
+	    continue;
 	gensio_pparm_unknown_parm(p, args[i]);
 	return GE_INVAL;
     }
@@ -1012,9 +1014,6 @@ struct stel_req {
     int maxval;
     gensio_control_done cdone;
     const struct stel_xlat_str *xlatstr;
-    void (*done)(struct sergensio *sio, int err, int val, void *cb_data);
-    void (*donesig)(struct sergensio *sio, int err, char *sig,
-		    unsigned int sig_len, void *cb_data);
     void *cb_data;
     int time_left;
     struct stel_req *next;
@@ -1022,7 +1021,6 @@ struct stel_req {
 
 struct stel_data {
     struct gensio *io;
-    struct sergensio *sio;
 
     struct gensio_os_funcs *o;
 
@@ -1111,10 +1109,6 @@ stel_queue(struct stel_data *sdata, int option,
 	   int minval, int maxval,
 	   gensio_control_done cdone,
 	   const struct stel_xlat_str *xlatstr,
-	   void (*done)(struct sergensio *sio, int err,
-			int baud, void *cb_data),
-	   void (*donesig)(struct sergensio *sio, int err, char *sig,
-			   unsigned int sig_len, void *cb_data),
 	   void *cb_data,
 	   gensio_time *timeout)
 {
@@ -1131,8 +1125,6 @@ stel_queue(struct stel_data *sdata, int option,
     req->option = option;
     req->cdone = cdone;
     req->xlatstr = xlatstr;
-    req->done = done;
-    req->donesig = donesig;
     req->cb_data = cb_data;
     req->minval = minval;
     if (!maxval)
@@ -1167,12 +1159,10 @@ stel_queue(struct stel_data *sdata, int option,
 static int
 stel_baud(struct stel_data *sdata, int baud, const char *sbaud,
 	  gensio_control_done cdone,
-	  void (*done)(struct sergensio *sio, int err,
-		       int baud, void *cb_data),
 	  void *cb_data,
 	  gensio_time *timeout)
 {
-    bool is_client = sergensio_is_client(sdata->sio);
+    bool is_client = gensio_is_client(sdata->io);
     unsigned char buf[6];
     int err;
 
@@ -1180,7 +1170,7 @@ stel_baud(struct stel_data *sdata, int baud, const char *sbaud,
 	baud = strtol(sbaud, NULL, 0);
 
     if (is_client) {
-	err = stel_queue(sdata, 1, 0, 0, cdone, NULL, done, NULL, cb_data,
+	err = stel_queue(sdata, 1, 0, 0, cdone, NULL, cb_data,
 			 timeout);
 	if (err)
 	    return err;
@@ -1209,13 +1199,11 @@ stel_queue_and_send(struct stel_data *sdata, int option, int val,
 		    int xmitbase, int minval, int maxval,
 		    gensio_control_done cdone,
 		    const struct stel_xlat_str *xlatstr,
-		    void (*done)(struct sergensio *sio, int err, int val,
-				 void *cb_data),
 		    void *cb_data,
 		    gensio_time *timeout)
 {
     unsigned char buf[3];
-    bool is_client = sergensio_is_client(sdata->sio);
+    bool is_client = gensio_is_client(sdata->io);
     int err;
 
     if (sval) {
@@ -1239,7 +1227,7 @@ stel_queue_and_send(struct stel_data *sdata, int option, int val,
 
     if (is_client) {
 	err = stel_queue(sdata, option, xmitbase, xmitbase + maxval,
-			 cdone, xlatstr, done, NULL, cb_data, timeout);
+			 cdone, xlatstr, cb_data, timeout);
 	if (err)
 	    return err;
     } else {
@@ -1257,57 +1245,51 @@ stel_queue_and_send(struct stel_data *sdata, int option, int val,
 static int
 stel_datasize(struct stel_data *sdata, int datasize, const char *sdatasize,
 	      gensio_control_done cdone,
-	      void (*done)(struct sergensio *sio, int err, int datasize,
-			   void *cb_data),
 	      void *cb_data,
 	      gensio_time *timeout)
 {
     return stel_queue_and_send(sdata, 2, datasize, sdatasize, 0, 0, 8,
-			       cdone, NULL, done, cb_data, timeout);
+			       cdone, NULL, cb_data, timeout);
 }
 
 static const struct stel_xlat_str stel_parity_xlatstr[] = {
     { "0", 0 },
     { "", 0 },
-    { "none", SERGENSIO_PARITY_NONE },
-    { "odd", SERGENSIO_PARITY_ODD },
-    { "even", SERGENSIO_PARITY_EVEN },
-    { "mark", SERGENSIO_PARITY_MARK },
-    { "space", SERGENSIO_PARITY_SPACE },
+    { "none", GENSIO_SER_PARITY_NONE },
+    { "odd", GENSIO_SER_PARITY_ODD },
+    { "even", GENSIO_SER_PARITY_EVEN },
+    { "mark", GENSIO_SER_PARITY_MARK },
+    { "space", GENSIO_SER_PARITY_SPACE },
     {}
 };
 
 static int
 stel_parity(struct stel_data *sdata, int parity, const char *sparity,
 	    gensio_control_done cdone,
-	    void (*done)(struct sergensio *sio, int err, int parity,
-			 void *cb_data),
 	    void *cb_data,
 	    gensio_time *timeout)
 {
     return stel_queue_and_send(sdata, 3, parity, sparity, 0, 0, 5,
-			       cdone, stel_parity_xlatstr, done, cb_data,
+			       cdone, stel_parity_xlatstr, cb_data,
 			       timeout);
 }
 
 static int
 stel_stopbits(struct stel_data *sdata, int stopbits, const char *sstopbits,
 	      gensio_control_done cdone,
-	      void (*done)(struct sergensio *sio, int err, int stopbits,
-			   void *cb_data),
 	      void *cb_data,
 	      gensio_time *timeout)
 {
     return stel_queue_and_send(sdata, 4, stopbits, sstopbits, 0, 0, 3,
-			       cdone, NULL, done, cb_data, timeout);
+			       cdone, NULL, cb_data, timeout);
 }
 
 static const struct stel_xlat_str stel_flow_xlatstr[] = {
     { "0", 0 },
     { "", 0 },
-    { "none", SERGENSIO_FLOWCONTROL_NONE },
-    { "xonxoff", SERGENSIO_FLOWCONTROL_XON_XOFF },
-    { "rtscts", SERGENSIO_FLOWCONTROL_RTS_CTS },
+    { "none", GENSIO_SER_FLOWCONTROL_NONE },
+    { "xonxoff", GENSIO_SER_FLOWCONTROL_XON_XOFF },
+    { "rtscts", GENSIO_SER_FLOWCONTROL_RTS_CTS },
     {}
 };
 
@@ -1315,23 +1297,21 @@ static int
 stel_flowcontrol(struct stel_data *sdata, int flowcontrol,
 		 const char *sflowcontrol,
 		 gensio_control_done cdone,
-		 void (*done)(struct sergensio *sio, int err,
-			      int flowcontrol, void *cb_data),
 		 void *cb_data,
 		 gensio_time *timeout)
 {
     return stel_queue_and_send(sdata, 5, flowcontrol, sflowcontrol, 0, 0, 3,
-			       cdone, stel_flow_xlatstr, done, cb_data,
+			       cdone, stel_flow_xlatstr, cb_data,
 			       timeout);
 }
 
 static const struct stel_xlat_str stel_iflow_xlatstr[] = {
     { "0", 0 },
     { "", 0 },
-    { "none", SERGENSIO_FLOWCONTROL_NONE },
-    { "dcd", SERGENSIO_FLOWCONTROL_DCD },
-    { "dtr", SERGENSIO_FLOWCONTROL_DTR },
-    { "dsr", SERGENSIO_FLOWCONTROL_DSR },
+    { "none", GENSIO_SER_FLOWCONTROL_NONE },
+    { "dcd", GENSIO_SER_FLOWCONTROL_DCD },
+    { "dtr", GENSIO_SER_FLOWCONTROL_DTR },
+    { "dsr", GENSIO_SER_FLOWCONTROL_DSR },
     {}
 };
 
@@ -1339,13 +1319,11 @@ static int
 stel_iflowcontrol(struct stel_data *sdata, int iflowcontrol,
 		  const char *siflowcontrol,
 		  gensio_control_done cdone,
-		  void (*done)(struct sergensio *sio, int err,
-			       int iflowcontrol, void *cb_data),
 		  void *cb_data,
 		  gensio_time *timeout)
 {
     return stel_queue_and_send(sdata, 5, iflowcontrol, siflowcontrol,13, 0, 6,
-			       cdone, stel_iflow_xlatstr, done, cb_data,
+			       cdone, stel_iflow_xlatstr, cb_data,
 			       timeout);
 }
 
@@ -1360,59 +1338,50 @@ static const struct stel_xlat_str stel_on_off_xlatstr[] = {
 static int
 stel_sbreak(struct stel_data *sdata, int breakv, const char *sbreakv,
 	    gensio_control_done cdone,
-	    void (*done)(struct sergensio *sio, int err, int breakv,
-			 void *cb_data),
 	    void *cb_data,
 	    gensio_time *timeout)
 {
     return stel_queue_and_send(sdata, 5, breakv, sbreakv, 4, 0, 2,
-			       cdone, stel_on_off_xlatstr, done, cb_data,
+			       cdone, stel_on_off_xlatstr, cb_data,
 			       timeout);
 }
 
 static int
 stel_dtr(struct stel_data *sdata, int dtr, const char *sdtr,
 	 gensio_control_done cdone,
-	 void (*done)(struct sergensio *sio, int err, int dtr,
-		      void *cb_data),
 	 void *cb_data,
 	 gensio_time *timeout)
 {
     return stel_queue_and_send(sdata, 5, dtr, sdtr, 7, 0, 2,
-			       cdone, stel_on_off_xlatstr, done, cb_data,
+			       cdone, stel_on_off_xlatstr, cb_data,
 			       timeout);
 }
 
 static int
 stel_rts(struct stel_data *sdata, int rts, const char *srts,
 	 gensio_control_done cdone,
-	 void (*done)(struct sergensio *sio, int err, int rts,
-		      void *cb_data),
 	 void *cb_data,
 	 gensio_time *timeout)
 {
     return stel_queue_and_send(sdata, 5, rts, srts, 10, 0, 2,
-			       cdone, stel_on_off_xlatstr, done, cb_data,
+			       cdone, stel_on_off_xlatstr, cb_data,
 			       timeout);
 }
 
 static int
 stel_signature(struct stel_data *sdata, const char *sig, unsigned int sig_len,
 	       gensio_control_done cdone,
-	       void (*done)(struct sergensio *sio, int err, char *sig,
-			    unsigned int sig_len, void *cb_data),
 	       void *cb_data,
 	       gensio_time *timeout)
 {
     unsigned char outopt[MAX_TELNET_CMD_XMIT_BUF];
-    bool is_client = sergensio_is_client(sdata->sio);
+    bool is_client = gensio_is_client(sdata->io);
 
     if (sig_len > (MAX_TELNET_CMD_XMIT_BUF - 2))
 	sig_len = MAX_TELNET_CMD_XMIT_BUF - 2;
 
     if (is_client) {
-	int err = stel_queue(sdata, 0, 0, 0, cdone, NULL, NULL, done, cb_data,
-			     timeout);
+	int err = stel_queue(sdata, 0, 0, 0, cdone, NULL, cb_data, timeout);
 	if (err)
 	    return err;
 
@@ -1439,7 +1408,7 @@ stel_send(struct stel_data *sdata, unsigned int opt, unsigned int val)
     buf[1] = opt;
     buf[2] = val;
 
-    if (!sergensio_is_client(sdata->sio))
+    if (!gensio_is_client(sdata->io))
 	buf[1] += 100;
 
     sdata->rops->send_option(sdata->filter, buf, 3);
@@ -1455,7 +1424,7 @@ stel_modemstate(struct stel_data *sdata, unsigned int val, const char *sval)
     if (sval)
 	val = strtol(sval, NULL, 0);
 
-    if (sergensio_is_client(sdata->sio))
+    if (gensio_is_client(sdata->io))
 	opt = 11;
     else
 	opt = 7;
@@ -1470,7 +1439,7 @@ stel_linestate(struct stel_data *sdata, unsigned int val, const char *sval)
     if (sval)
 	val = strtol(sval, NULL, 0);
 
-    if (sergensio_is_client(sdata->sio))
+    if (gensio_is_client(sdata->io))
 	opt = 10;
     else
 	opt = 6;
@@ -1490,8 +1459,6 @@ stel_send_modemstate(struct stel_data *sdata, unsigned int val,
 static int
 stel_set_modemstate_mask(struct stel_data *sdata, unsigned int val,
 			 const char *sval, gensio_control_done cdone,
-			 void (*done)(struct sergensio *sio, int err,
-				      int datasize, void *cb_data),
 			 void *cb_data,
 			 gensio_time *timeout)
 {
@@ -1499,7 +1466,7 @@ stel_set_modemstate_mask(struct stel_data *sdata, unsigned int val,
 	val = strtol(sval, NULL, 0);
 
     return stel_queue_and_send(sdata, 11, val, NULL, 0, 0, 255, cdone,
-			       NULL, done, cb_data, timeout);
+			       NULL, cb_data, timeout);
 }
 
 static int
@@ -1513,8 +1480,6 @@ stel_send_linestate(struct stel_data *sdata, unsigned int val, const char *sval)
 static int
 stel_set_linestate_mask(struct stel_data *sdata, unsigned int val,
 			const char *sval, gensio_control_done cdone,
-			void (*done)(struct sergensio *sio, int err,
-				     int datasize, void *cb_data),
 			void *cb_data,
 			gensio_time *timeout)
 {
@@ -1522,7 +1487,7 @@ stel_set_linestate_mask(struct stel_data *sdata, unsigned int val,
 	val = strtol(sval, NULL, 0);
 
     return stel_queue_and_send(sdata, 10, val, NULL, 0, 0, 255, cdone,
-			       NULL, done, cb_data, timeout);
+			       NULL, cb_data, timeout);
 }
 
 static int
@@ -1545,7 +1510,7 @@ stel_flowcontrol_state(struct stel_data *sdata, bool val, const char *sval)
 	buf[1] = 8;
     else
 	buf[1] = 9;
-    if (!sergensio_is_client(sdata->sio))
+    if (!gensio_is_client(sdata->io))
 	buf[1] += 100;
 
     sdata->rops->send_option(sdata->filter, buf, 2);
@@ -1565,13 +1530,11 @@ static const struct stel_xlat_str stel_flush_xlatstr[] = {
 static int
 stel_flush(struct stel_data *sdata, unsigned int val, const char *sval,
 	   gensio_control_done cdone,
-	   void (*done)(struct sergensio *sio, int err, int datasize,
-			void *cb_data),
 	   void *cb_data,
 	   gensio_time *timeout)
 {
     return stel_queue_and_send(sdata, 12, val, sval, 0, 0, 3, cdone,
-			       stel_flush_xlatstr, done, cb_data, timeout);
+			       stel_flush_xlatstr, cb_data, timeout);
 }
 
 static int
@@ -1585,63 +1548,6 @@ stel_send_break(struct stel_data *sdata)
     return 0;
 }
 
-static int
-sergensio_stel_func(struct sergensio *sio, int op, int val, char *buf,
-		    void *done, void *cb_data)
-{
-    struct stel_data *sdata = sergensio_get_gensio_data(sio);
-
-    switch (op) {
-    case SERGENSIO_FUNC_BAUD:
-	return stel_baud(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_DATASIZE:
-	return stel_datasize(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_PARITY:
-	return stel_parity(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_STOPBITS:
-	return stel_stopbits(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_FLOWCONTROL:
-	return stel_flowcontrol(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_IFLOWCONTROL:
-	return stel_iflowcontrol(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_SBREAK:
-	return stel_sbreak(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_DTR:
-	return stel_dtr(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_RTS:
-	return stel_rts(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_MODEMSTATE:
-	return stel_modemstate(sdata, val, NULL);
-
-    case SERGENSIO_FUNC_LINESTATE:
-	return stel_linestate(sdata, val, NULL);
-
-    case SERGENSIO_FUNC_FLOWCONTROL_STATE:
-	return stel_flowcontrol_state(sdata, val, NULL);
-
-    case SERGENSIO_FUNC_FLUSH:
-	return stel_flush(sdata, val, NULL, NULL, done, cb_data, NULL);
-
-    case SERGENSIO_FUNC_SIGNATURE:
-	return stel_signature(sdata, buf, val, NULL, NULL, NULL, NULL);
-
-    case SERGENSIO_FUNC_SEND_BREAK:
-	return stel_send_break(sdata);
-
-    default:
-	return GE_NOTSUP;
-    }
-}
-
 static int
 stel_control(void *handler_data, bool get, int option,
 	     char *data, gensiods *datalen)
@@ -1668,7 +1574,7 @@ stel_control(void *handler_data, bool get, int option,
 	return stel_flowcontrol_state(sdata, 0, data);
 
     case GENSIO_CONTROL_SER_FLUSH:
-	return stel_flush(sdata, 0, data, NULL, NULL, NULL, NULL);
+	return stel_flush(sdata, 0, data, NULL, NULL, NULL);
 
     case GENSIO_CONTROL_SER_SEND_BREAK:
 	return stel_send_break(sdata);
@@ -1687,52 +1593,58 @@ stel_acontrol(void *handler_data, bool get, int option,
     gensio_control_done cdone = idata->done;
     void *cb_data = idata->cb_data;
     gensio_time *timeout = idata->timeout;
+    gensiods datalen = idata->datalen;
 
     if (!gensio_is_serial(sdata->io))
 	return GE_NOTSUP;
 
+    if (get)
+	data = NULL;
+
     switch (option) {
     case GENSIO_ACONTROL_SER_BAUD:
-	return stel_baud(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_baud(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_DATASIZE:
-	return stel_datasize(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_datasize(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_PARITY:
-	return stel_parity(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_parity(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_STOPBITS:
-	return stel_stopbits(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_stopbits(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_FLOWCONTROL:
-	return stel_flowcontrol(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_flowcontrol(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_IFLOWCONTROL:
-	return stel_iflowcontrol(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_iflowcontrol(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_SBREAK:
-	return stel_sbreak(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_sbreak(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_DTR:
-	return stel_dtr(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_dtr(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_RTS:
-	return stel_rts(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_rts(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_FLUSH:
-	return stel_flush(sdata, 0, data, cdone, NULL, cb_data, timeout);
+	return stel_flush(sdata, 0, data, cdone, cb_data, timeout);
 
     case GENSIO_ACONTROL_SER_SET_MODEMSTATE_MASK:
-	return stel_set_modemstate_mask(sdata, 0, data, cdone, NULL, cb_data,
+	return stel_set_modemstate_mask(sdata, 0, data, cdone, cb_data,
 					timeout);
 
     case GENSIO_ACONTROL_SER_SET_LINESTATE_MASK:
-	return stel_set_linestate_mask(sdata, 0, data, cdone, NULL, cb_data,
+	return stel_set_linestate_mask(sdata, 0, data, cdone, cb_data,
 				       timeout);
 
     case GENSIO_ACONTROL_SER_SIGNATURE:
-	return stel_signature(sdata, data, idata->datalen,
-			      cdone, NULL, cb_data, timeout);
+	if (get)
+	    datalen = 0;
+	return stel_signature(sdata, data, datalen,
+			      cdone, cb_data, timeout);
 
     default:
 	return GE_NOTSUP;
@@ -1870,13 +1782,6 @@ stelc_com_port_cmd(void *handler_data, const unsigned char *option,
 		curr->cdone(sdata->io, 0, sval, strlen(sval), curr->cb_data);
 	    }
 	}
-	if (sig) {
-	    if (curr->donesig)
-		curr->donesig(sdata->sio, 0, sig, sig_len, curr->cb_data);
-	} else {
-	    if (curr->done)
-		curr->done(sdata->sio, 0, val - curr->minval, curr->cb_data);
-	}
 	sdata->o->free(sdata->o, curr);
 	return;
     }
@@ -1948,10 +1853,6 @@ stelc_timeout(void *handler_data)
     while (req) {
 	if (req->cdone)
 	    req->cdone(sdata->io, GE_TIMEDOUT, NULL, 0, req->cb_data);
-	else if (req->done)
-	    req->done(sdata->sio, GE_TIMEDOUT, 0, req->cb_data);
-	else if (req->donesig)
-	    req->donesig(sdata->sio, GE_TIMEDOUT, NULL, 0, req->cb_data);
 	prev = req;
 	req = req->next;
 	sdata->o->free(sdata->o, prev);
@@ -1969,8 +1870,6 @@ stel_free(void *handler_data)
 {
     struct stel_data *sdata = handler_data;
 
-    if (sdata->sio)
-	sergensio_data_free(sdata->sio);
     if (sdata->lock)
 	sdata->o->free_lock(sdata->lock);
     while (sdata->reqs) {
@@ -2267,7 +2166,8 @@ struct gensio_telnet_filter_callbacks sergensio_telnet_server_filter_cbs = {
 static int
 stel_setup(struct gensio_pparm_info *p,
 	   const char * const args[], bool default_is_client,
-	   struct gensio_os_funcs *o, struct stel_data **rsdata)
+	   struct gensio_os_funcs *o, struct gensio_base_parms *parms,
+	   struct stel_data **rsdata)
 {
     struct stel_data *sdata;
     unsigned int i;
@@ -2319,7 +2219,8 @@ stel_setup(struct gensio_pparm_info *p,
 				     (is_client ?
 				      &sergensio_telnet_filter_cbs :
 				      &sergensio_telnet_server_filter_cbs),
-				     sdata, &sdata->rops, &sdata->filter);
+				     sdata, &sdata->rops, parms,
+				     &sdata->filter);
     if (err)
 	goto out_err;
 
@@ -2342,10 +2243,11 @@ stel_setup(struct gensio_pparm_info *p,
 }
 
 static int
-telnet_gensio_alloc(struct gensio *child, const char * const args[],
-		    struct gensio_os_funcs *o,
-		    gensio_event cb, void *user_data,
-		    struct gensio **rio)
+telnet_gensio_alloc2(struct gensio *child, const char * const args[],
+		     struct gensio_os_funcs *o,
+		     gensio_event cb, void *user_data,
+		     struct gensio_base_parms *parms,
+		     struct gensio **rio)
 {
     struct stel_data *sdata;
     struct gensio_ll *ll = NULL;
@@ -2353,9 +2255,15 @@ telnet_gensio_alloc(struct gensio *child, const char * const args[],
     int err;
     GENSIO_DECLARE_PPGENSIO(p, o, cb, "telnet", user_data);
 
-    err = stel_setup(&p, args, true, o, &sdata);
+    if (!parms) {
+	err = gensio_base_parms_alloc(o, true, "telnet", &parms);
+	if (err)
+	    goto out_err2;
+    }
+
+    err = stel_setup(&p, args, true, o, parms, &sdata);
     if (err)
-	return err;
+	goto out_err2;
 
     ll = gensio_gensio_ll_alloc(o, child);
     if (!ll)
@@ -2367,15 +2275,14 @@ telnet_gensio_alloc(struct gensio *child, const char * const args[],
     if (!io)
 	goto out_nomem;
 
+    err = gensio_base_parms_set(io, &parms);
+    if (err)
+	goto out_err;
+
     sdata->io = io;
 
-    if (sdata->allow_rfc2217) {
+    if (sdata->allow_rfc2217)
 	gensio_set_is_serial(io, true);
-	err = sergensio_addclass(o, io, sergensio_stel_func, sdata,
-				 &sdata->sio);
-	if (err)
-	    goto out_err;
-    }
 
     gensio_free(child); /* Lose the ref we acquired. */
     gensio_set_is_client(io, sdata->is_client);
@@ -2396,9 +2303,21 @@ telnet_gensio_alloc(struct gensio *child, const char * const args[],
 	if (ll)
 	    gensio_ll_free(ll);
     }
+ out_err2:
+    if (parms)
+	gensio_base_parms_free(&parms);
     return err;
 }
 
+static int
+telnet_gensio_alloc(struct gensio *child, const char * const args[],
+		    struct gensio_os_funcs *o,
+		    gensio_event cb, void *user_data,
+		    struct gensio **rio)
+{
+    return telnet_gensio_alloc2(child, args, o, cb, user_data, NULL, rio);
+}
+
 static int
 str_to_telnet_gensio(const char *str, const char * const args[],
 		     struct gensio_os_funcs *o,
@@ -2421,7 +2340,7 @@ str_to_telnet_gensio(const char *str, const char * const args[],
 }
 
 struct stela_data {
-    struct sergensio_accepter *sacc;
+    struct gensio_accepter *acc;
 
     gensiods max_read_size;
     gensiods max_write_size;
@@ -2440,7 +2359,7 @@ stela_free(void *acc_data)
 {
     struct stela_data *stela = acc_data;
 
-    /* stela->sacc will be freed in the class callback. */
+    /* stela->acc will be freed in the class callback. */
     stela->o->free(stela->o, stela);
 }
 
@@ -2458,9 +2377,14 @@ stela_alloc_gensio(void *acc_data, const char * const *iargs,
     gensiods max_write_size = stela->max_write_size;
     gensiods max_read_size = stela->max_read_size;
     bool is_client = stela->is_client;
+    struct gensio_base_parms *parms = NULL;
     GENSIO_DECLARE_PPACCEPTER(p, stela->o, stela->cb, "telnet",
 			      stela->user_data);
 
+    parms = gensio_acc_base_parms_dup(stela->acc);
+    if (!parms)
+	return GE_NOMEM;
+
     for (i = 0; iargs && iargs[i]; i++) {
 	if (gensio_pparm_bool(&p, iargs[i], "rfc2217", &allow_rfc2217) > 0)
 	    continue;
@@ -2473,7 +2397,10 @@ stela_alloc_gensio(void *acc_data, const char * const *iargs,
 	if (gensio_pparm_boolv(&p, iargs[i], "mode", "client", "server",
 			       &is_client) > 0)
 	    continue;
+	if (gensio_base_parm(parms, &p, iargs[i]) > 0)
+	    continue;
 	gensio_pparm_unknown_parm(&p, iargs[i]);
+	gensio_base_parms_free(&parms);
 	return GE_INVAL;
     }
 
@@ -2495,7 +2422,7 @@ stela_alloc_gensio(void *acc_data, const char * const *iargs,
     if (!is_client)
 	args[i++] = "mode=server";
 
-    return telnet_gensio_alloc(child, args, o, NULL, NULL, rio);
+    return telnet_gensio_alloc2(child, args, o, NULL, NULL, parms, rio);
 }
 
 static int
@@ -2520,7 +2447,7 @@ stela_new_child(void *acc_data, void **finish_data,
     snprintf(arg5, sizeof(arg5), "mode=%s",
 	     stela->is_client ? "client" : "server");
 
-    err = stel_setup(&p, args, false, o, &sdata);
+    err = stel_setup(&p, args, false, o, NULL, &sdata);
     if (err)
 	return err;
 
@@ -2535,17 +2462,17 @@ stela_finish_parent(void *acc_data, void *finish_data, struct gensio *io,
 		    struct gensio *child)
 {
     struct stel_data *sdata = finish_data;
+    struct stela_data *stela = acc_data;
     int err;
 
+    err = gensio_acc_base_parms_apply(stela->acc, io);
+    if (err)
+      return err;
+
     sdata->io = io;
 
-    if (sdata->allow_rfc2217) {
+    if (sdata->allow_rfc2217)
 	gensio_set_is_serial(io, true);
-	err = sergensio_addclass(sdata->o, io, sergensio_stel_func, sdata,
-				 &sdata->sio);
-	if (err)
-	    return err;
-    }
 
     gensio_set_is_client(io, sdata->is_client);
 
@@ -2575,14 +2502,6 @@ gensio_gensio_acc_telnet_cb(void *acc_data, int op, void *data1, void *data2,
     }
 }
 
-static int
-sergensio_stela_func(struct sergensio_accepter *sacc,
-		     int op, int val, char *buf,
-		     void *done, void *cb_data)
-{
-    return GE_NOTSUP;
-}
-
 static int
 telnet_gensio_accepter_alloc(struct gensio_accepter *child,
 			     const char * const args[],
@@ -2591,7 +2510,6 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
 			     struct gensio_accepter **raccepter)
 {
     struct stela_data *stela;
-    int err;
     unsigned int i;
     gensiods max_read_size = GENSIO_DEFAULT_BUF_SIZE;
     gensiods max_write_size = GENSIO_DEFAULT_BUF_SIZE;
@@ -2600,18 +2518,23 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
     bool is_client = false;
     struct gensio_accepter *accepter = NULL;
     int rv, ival;
+    struct gensio_base_parms *parms;
     GENSIO_DECLARE_PPACCEPTER(p, o, cb, "telnet", user_data);
 
+    rv = gensio_base_parms_alloc(o, true, "telnet", &parms);
+    if (rv)
+	goto out_err2;
+
     rv = gensio_get_default(o, "telnet", "rfc2217", false,
 			    GENSIO_DEFAULT_BOOL, NULL, &ival);
     if (rv)
-	return rv;
+	goto out_err2;
     allow_rfc2217 = ival;
 
     rv = gensio_get_default(o, "telnet", "winsize", false,
 			    GENSIO_DEFAULT_BOOL, NULL, &ival);
     if (rv)
-	return rv;
+	goto out_err2;
     allow_rfc1073 = ival;
 
     for (i = 0; args && args[i]; i++) {
@@ -2626,13 +2549,18 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
 	if (gensio_pparm_boolv(&p, args[i], "mode", "client", "server",
 			       &is_client) > 0)
 	    continue;
+	if (gensio_base_parm(parms, &p, args[i]) > 0)
+	    continue;
 	gensio_pparm_unknown_parm(&p, args[i]);
-	return GE_INVAL;
+	rv = GE_INVAL;
+	goto out_err2;
     }
 
     stela = o->zalloc(o, sizeof(*stela));
-    if (!stela)
-	return GE_NOMEM;
+    if (!stela) {
+	rv = GE_NOMEM;
+	goto out_err2;
+    }
 
     stela->o = o;
     stela->cb = cb;
@@ -2643,20 +2571,21 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
     stela->allow_rfc1073 = allow_rfc1073;
     stela->is_client = is_client;
 
-    err = gensio_gensio_accepter_alloc(child, o, "telnet",
+    rv = gensio_gensio_accepter_alloc(child, o, "telnet",
 				       cb, user_data,
 				       gensio_gensio_acc_telnet_cb, stela,
 				       &accepter);
-    if (err)
+    if (rv)
 	goto out_err;
 
-    if (allow_rfc2217) {
+    rv = gensio_acc_base_parms_set(accepter, &parms);
+    if (rv)
+	goto out_err;
+    
+    if (allow_rfc2217)
 	gensio_acc_set_is_serial(accepter, true);
-	err = sergensio_acc_addclass(o, accepter, sergensio_stela_func, stela,
-				     &stela->sacc);
-	if (err)
-	    goto out_err;
-    }
+
+    stela->acc = accepter;
     gensio_acc_set_is_reliable(accepter, gensio_acc_is_reliable(child));
 
     *raccepter = accepter;
@@ -2668,7 +2597,10 @@ telnet_gensio_accepter_alloc(struct gensio_accepter *child,
 	gensio_gensio_acc_free_nochild(accepter);
     else
 	stela_free(stela);
-    return err;
+ out_err2:
+    if (parms)
+	gensio_base_parms_free(&parms);
+    return rv;
 }
 
 static int
