@@ -24,7 +24,8 @@
 #define TN_OPT_BINARY_TRANSMISSION	0
 #define TN_OPT_ECHO			1
 #define TN_OPT_SUPPRESS_GO_AHEAD	3
-#define TN_OPT_COM_PORT			44
+#define TN_OPT_COM_PORT			44 /* rfc2217 */
+#define TN_OPT_NAWS			31 /* rfc1073 - Window size */
 
 typedef struct telnet_data_s telnet_data_t;
 
@@ -95,7 +96,8 @@ struct telnet_data_s
 
 /* Send a telnet command.  This will set td->error to true if an
    output error occurs (out of space). */
-void telnet_cmd_send(telnet_data_t *td, const unsigned char *cmd, int len);
+void telnet_cmd_send(telnet_data_t *td, const unsigned char *cmd,
+		     unsigned int len);
 
 /* Received some data from the TCP port representing telnet, process
    it.  The leftover length is returned by this function, and the
