@@ -14,6 +14,7 @@
 
 #include <gensio/gensio.h>
 #include <gensio/gensio_list.h>
+#include <gensio/gensio_refcount.h>
 #include "utils.h"
 
 char *
@@ -839,10 +840,7 @@ gensio_log_level_to_str(enum gensio_log_levels level)
 }
 
 struct gensio_cntstr {
-    int refcount;
-#if !HAVE_GCC_ATOMICS
-    struct gensio_lock *lock;
-#endif
+    gensio_refcount refcount;
     char *str;
 };
 
@@ -860,7 +858,7 @@ gensio_cntstr_make(struct gensio_os_funcs *o, const char *src,
     str = o->zalloc(o, len + sizeof(*str));
     if (!str)
 	return GE_NOMEM;
-    str->refcount = 1;
+    gensio_refcount_init(o, &str->refcount, 1);
     if (src) {
 	str->str = ((char *) str) + sizeof(*str);
 	strcpy(str->str, src);
@@ -872,32 +870,19 @@ gensio_cntstr_make(struct gensio_os_funcs *o, const char *src,
 gensio_cntstr *
 gensio_cntstr_ref(struct gensio_os_funcs *o, gensio_cntstr *str)
 {
-#if HAVE_GCC_ATOMICS
-    __atomic_add_fetch(&str->refcount, 1, __ATOMIC_SEQ_CST);
-#else
-    o->lock(str->lock);
-    str->refcount++;
-    o->unlock(str->lock);
-#endif
+    gensio_refcount_inc(&str->refcount);
     return str;
 }
 
 void
 gensio_cntstr_free(struct gensio_os_funcs *o, gensio_cntstr *str)
 {
-    assert(str->refcount > 0);
-#if HAVE_GCC_ATOMICS
-    if (__atomic_sub_fetch(&str->refcount, 1, __ATOMIC_SEQ_CST) == 0) {
+    unsigned int newval = gensio_refcount_dec(&str->refcount);
+
+    if (newval == 0) {
+	gensio_refcount_cleanup(&str->refcount);
 	o->free(o, str);
-	return;
     }
-#else
-    o->lock(str->lock);
-    str->refcount--;
-    if (str->refcount == 0)
-	o->free(o, str);
-    o->unlock(str->lock);
-#endif
 }
 
 int
@@ -916,7 +901,7 @@ gensio_cntstr_vsprintf(struct gensio_os_funcs *o, gensio_cntstr **dest,
 	va_end(va2);
 	return GE_NOMEM;
     }
-    str->refcount = 1;
+    gensio_refcount_init(o, &str->refcount, 1);
     str->str = ((char *) str) + sizeof(*str);
     vsnprintf(str->str, len, fmt, va2);
     va_end(va2);
