@@ -13,16 +13,24 @@
 
 #include <gensio/gensio_dllvisibility.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #ifdef SWIGPYTHON
 typedef PyObject swig_cb;
-#endif 
+#endif
 
 GENSIO_DLL_PUBLIC
-void gensio_swig_setup_os_funcs(struct gensio_os_funcs *o,
-				swig_cb *log_handler);
+int gensio_swig_setup_os_funcs(struct gensio_os_funcs *o,
+			       swig_cb *log_handler);
 
 GENSIO_DLL_PUBLIC
 void check_os_funcs_free(struct gensio_os_funcs *o);
 
 GENSIO_DLL_PUBLIC
 int get_os_funcs_refcount(struct gensio_os_funcs *o);
+
+#ifdef __cplusplus
+}
+#endif
