@@ -67,16 +67,17 @@ static void *
 gensio_tcl_zalloc(struct gensio_os_funcs *f, gensiods size)
 {
     struct gensio_data *d = f->user_data;
-
-    return gensio_i_zalloc(d->mtrack, size);
+    TRACE_MEM;
+    return gensio_i_zalloc(d->mtrack, size,
+			   TRACE_MEM_CALLERS, TRACE_MEM_CALLERS_SIZE);
 }
 
 static void
 gensio_tcl_free(struct gensio_os_funcs *f, void *data)
 {
     struct gensio_data *d = f->user_data;
-
-    gensio_i_free(d->mtrack, data);
+    TRACE_MEM;
+    gensio_i_free(d->mtrack, data, TRACE_MEM_CALLERS, TRACE_MEM_CALLERS_SIZE);
 }
 
 struct gensio_lock {
@@ -128,6 +129,7 @@ struct gensio_iod_tcl {
 
     int orig_fd;
     int fd;
+    int sfd;
     enum gensio_iod_type type;
     void *sockinfo;
     bool handlers_set;
@@ -880,11 +882,11 @@ gensio_tcl_wake(struct gensio_waiter *w)
 
 static int
 gensio_tcl_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
-		   intptr_t ofd, struct gensio_iod **riod)
+		   intptr_t ofd, struct gensio_iod **riod, ...)
 {
     struct gensio_iod_tcl *iod = NULL;
     bool closefd = false;
-    int err = GE_NOMEM, fd = ofd;
+    int err = GE_NOMEM, fd = ofd, sfd = -1;
 
     if (type == GENSIO_IOD_CONSOLE) {
        if (fd == 0)
@@ -897,7 +899,7 @@ gensio_tcl_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
            return gensio_os_err_to_err(o, errno);
        closefd = true;
     } else if (type == GENSIO_IOD_PTY) {
-	err = gensio_unix_pty_alloc(o, &fd);
+	err = gensio_unix_pty_alloc(o, &fd, &sfd);
 	if (err)
 	    return err;
 	closefd = true;
@@ -911,6 +913,7 @@ gensio_tcl_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
 
     iod->r.f = o;
     iod->fd = fd;
+    iod->sfd = sfd;
     iod->orig_fd = ofd;
     if (type == GENSIO_IOD_STDIO) {
 	struct stat statb;
@@ -948,8 +951,11 @@ gensio_tcl_add_iod(struct gensio_os_funcs *o, enum gensio_iod_type type,
  out_err:
     if (iod)
 	o->free(o, iod);
-    else if (closefd)
+    else if (closefd) {
 	close(fd);
+	if (sfd != -1)
+	    close(sfd);
+    }
     return err;
 }
 
@@ -1026,7 +1032,7 @@ gensio_tcl_pty_control(struct gensio_iod_tcl *iod, int op, bool get,
 	return 0;
 
     case GENSIO_IOD_CONTROL_START:
-	return gensio_unix_pty_start(o, iod->fd, iod->argv,
+	return gensio_unix_pty_start(o, iod->fd, &iod->sfd, iod->argv,
 				     iod->env, iod->start_dir, &iod->pid);
 
     case GENSIO_IOD_CONTROL_STOP:
@@ -1145,6 +1151,10 @@ gensio_tcl_close(struct gensio_iod **iodp)
 	/* Close should never fail, but don't crash in production builds. */
 	    assert(err == 0);
 #endif
+	    if (iod->sfd != -1) {
+		close(iod->sfd);
+		iod->sfd = -1;
+	    }
 	}
     }
     o->release_iod(iiod);
@@ -1423,22 +1433,6 @@ gensio_tcl_free_funcs(struct gensio_os_funcs *f)
     free(f);
 }
 
-static Tcl_Mutex once_lock;
-
-static void
-gensio_tcl_call_once(struct gensio_os_funcs *f, struct gensio_once *once,
-		      void (*func)(void *cb_data), void *cb_data)
-{
-    if (once->called)
-	return;
-    Tcl_MutexLock(&once_lock);
-    if (!once->called) {
-	once->called = true;
-	func(cb_data);
-    }
-    Tcl_MutexUnlock(&once_lock);
-}
-
 static void
 gensio_tcl_get_monotonic_time(struct gensio_os_funcs *f, gensio_time *time)
 {
@@ -1559,7 +1553,7 @@ gensio_tcl_funcs_alloc(struct gensio_os_funcs **ro)
     o->service = gensio_tcl_service;
     o->get_funcs = gensio_tcl_get_funcs;
     o->free_funcs = gensio_tcl_free_funcs;
-    o->call_once = gensio_tcl_call_once;
+    o->call_once = gensio_call_once;
     o->get_monotonic_time = gensio_tcl_get_monotonic_time;
     o->handle_fork = gensio_tcl_handle_fork;
     o->add_iod = gensio_tcl_add_iod;
