@@ -47,6 +47,20 @@
 
 unsigned int debug;
 
+/*
+ * Permissions are not easily fixable on msys, so checking for private
+ * doesn't work ATM.  It always reads 755 for directories and 644 for
+ * files.  You can fix it with icacls, but it still won't display
+ * properly.
+ */
+#ifdef __MSYS__
+static bool do_check_private = false;
+#else
+static bool do_check_private = true;
+#endif
+
+static bool nointeractive_pw = false;
+
 struct gdata {
     struct gensio_os_funcs *o;
     struct gensio_waiter *waiter;
@@ -180,7 +194,7 @@ setup_promptuser(struct gdata *ginfo, const char *prompt, struct gensio **rtty)
     int err;
     const char *constr = "stdio(console,raw)";
 
-    if (aux_data.flags & GTLSSH_AUX_FLAG_NO_INTERACTIVE)
+    if (aux_data.flags & GTLSSH_AUX_FLAG_NO_INTERACTIVE && !nointeractive_pw)
 	return GE_KEYNOTFOUND;
 
     err = str_to_gensio(constr, ginfo->o, NULL, NULL, &tty);
@@ -382,6 +396,7 @@ help(int err)
     printf("    for detail on how to do regex, glob, etc.\n");
     printf("  --2fa <str> - Pass the given string as 2-factor auth data.\n");
     printf("  --nointeractive - Do not do interactive login queries.\n");
+    printf("  --nointeractive-pw - Allow pw prompt even with nointeractive.\n");
     printf("  -d, --debug - Enable debug.  Specify more than once to increase\n"
 	   "    the debug level\n");
     printf("  -L <accept addr>:<connect addr> - Listen at the <accept addr>\n"
@@ -741,7 +756,7 @@ lookup_certinfo(struct gensio_os_funcs *o,
     if (err)
 	goto out_err;
 
-    err = checkout_file(glogger, NULL, keyname, false, true);
+    err = checkout_file(glogger, NULL, keyname, false, do_check_private);
     if (err)
 	goto out_err;
 
@@ -1438,7 +1453,7 @@ handle_port(struct gensio_os_funcs *o, bool remote, const char *iaddr)
     const char *type = NULL;
     unsigned int num_s = 0, pos = 0;
     char *connecter_str = NULL, *accepter_str = NULL;
-    char *addr = strdup(iaddr);
+    char *addr = strdup(iaddr), *addr2;
     int err = -1;
     bool has_bind = false;
 
@@ -1447,11 +1462,12 @@ handle_port(struct gensio_os_funcs *o, bool remote, const char *iaddr)
 	goto out_err;
     }
 
+    addr2 = addr;
     while (num_s < 4) {
-	s[num_s++] = addr;
-	addr = strchr(addr, ':');
-	if (addr)
-	    *addr++ = '\0';
+	s[num_s++] = addr2;
+	addr2 = strchr(addr2, ':');
+	if (addr2)
+	    *addr2++ = '\0';
 	else
 	    break;
     }
@@ -1925,6 +1941,9 @@ main(int argc, char *argv[])
 	} else if ((err = cmparg(argc, argv, &arg, NULL, "--nointeractive",
 				 NULL))) {
 	    aux_data.flags |= GTLSSH_AUX_FLAG_NO_INTERACTIVE;
+	} else if ((err = cmparg(argc, argv, &arg, NULL, "--nointeractive-pw",
+				 NULL))) {
+	    nointeractive_pw = true;
 	} else if ((err = cmparg(argc, argv, &arg, NULL, "--privileged",
 				 NULL))) {
 	    aux_data.flags |= GTLSSH_AUX_FLAG_PRIVILEGED;
@@ -2006,8 +2025,12 @@ main(int argc, char *argv[])
 	unsigned int svclen = 0;
 
 	/* User gave us a remote program. */
-	for (i = arg; i < argc; i++)
+	for (i = arg; i < argc; i++) {
 	    svclen += strlen(argv[i]) + 1; /* Extra space for nil at end */
+	    if (argv[i][0] == '\0' || argv[i][0] == '\xff')
+		/* See note below on why we do this. */
+		svclen += 1;
+	}
 	svclen += 9; /* Space for "program:" and final nil. */
 	/* Note that ending '\0' is handled by final space. */
 
@@ -2019,6 +2042,15 @@ main(int argc, char *argv[])
 	strcpy(service, "program:");
 	svclen = 8;
 	for (i = arg; i < argc; i++) {
+	    if (argv[i][0] == '\0' || argv[i][0] == '\xff')
+		/*
+		 * Send a "\xff" as the first character for an empty
+		 * string.  Otherwize and empty string will look like
+		 * the \0\0 used to mark the end.  Also add a '\xff"
+		 * for a parameter starting with 0xff.  We will drop
+		 * it on the other side.
+		 */
+		service[svclen++] = '\xff';
 	    svclen += sprintf(service + svclen, "%s", argv[i]);
 	    service[svclen++] = '\0';
 	}
@@ -2063,7 +2095,7 @@ main(int argc, char *argv[])
 	    return 1;
     }
 
-    err = checkout_file(glogger, NULL, tlssh_dir, true, true);
+    err = checkout_file(glogger, NULL, tlssh_dir, true, do_check_private);
     if (err)
 	return 1;
 
